﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using UHF_Reader.Public;
using System.Threading;

namespace UHF_Reader
{
    public partial class FmUpdate : Form
    {
        public FmUpdate()
        {
            InitializeComponent();
        }

        public delegate void AddShow(string txt);
        public delegate void PbBar(int step);

        private void FmUpdate_Load(object sender, EventArgs e)
        {
            string[] serials = new string[32];

            int hidCnt = PublicFunction.SearchHids(ref serials);

            if (hidCnt == 0)
            {
                return;
            }
            else
            {
                for (int i = 0; i < hidCnt; i++)
                {
                    CbCom.Items.Add(serials[i]);
                }
            }

            if (CbCom.Items.Count != 0)
            {
                CbCom.SelectedIndex = 0;
            }
            else
            {
                MessageBox.Show("未搜索到HID");
            }
        }

        private void BtScan_Click(object sender, EventArgs e)
        {
            if (OpenFileDialogBin.ShowDialog() == DialogResult.OK)
            {
                this.TbBin.Text = OpenFileDialogBin.FileName;
            }
        }

        private void BtUpdate_Click(object sender, EventArgs e)
        {
            if (CbCom.Items.Count == 0)
            {
                MessageBox.Show("未搜索到HID");
                return;
            }
            string port = CbCom.SelectedItem.ToString();
            if (port == "")
            {
                MessageBox.Show("请选择设备");
                return;
            }

            PublicFunction.SelectBaudAndCrc(57600, 0x01);
            PublicFunction.DisConnectRLM();

            string binPath = TbBin.Text.ToString().Trim();
            if (binPath.Length == 0)
            {
                MessageBox.Show("请选择升级文件");
                return;
            }

            if (!File.Exists(binPath))
            {
                MessageBox.Show("文件不存在");
                return;
            }
            
            int file_byte_size = 0;
            
            FileInfo File_info = new FileInfo(TbBin.Text);
            file_byte_size = (int)File_info.Length;//获取文件大小  字节
            
            PBUpdate.Maximum = file_byte_size;
            PBUpdate.Value = 0;
            PBUpdate.Refresh();

            byte[] RN32 = new byte[4];


            if (!PublicFunction.StarUpdate(port, RN32))
            {
                MessageBox.Show("升级通知失败");
                return;
            }
            AddTxt("发送升级文件长度成功");
            System.Threading.Thread.Sleep(10);

            if (!PublicFunction.SendRN(RN32))
            {
                MessageBox.Show("发送RN32反码失败");
                return;
            }
            AddTxt("发送RN32反码成功");
            
            Thread thread = new Thread(new ParameterizedThreadStart(this.UpdateSendPackThread));
            thread.IsBackground = true;
            thread.Start(binPath);
        }


        private void UpdateSendPackThread(object obj)
        {
            string binPath = (string)obj;
            FileInfo File_info = new FileInfo(binPath);

            byte[] package = new byte[1024];
            int file_byte_size = (int)(File_info.Length);   //获取文件大小（字节数）
            int package_num = file_byte_size / 1024;         //获取完整512字节数据包个数
            int lastpack_len = file_byte_size % 1024;        //获取最后一个数据包长度（字节数）
            int i = 0;

            PbBar updateBarByStep = new PbBar(UpdateProgressBarByStep);
            AddShow addShow = new AddShow(AddTxt);

            FileStream filestream = new FileStream(binPath, FileMode.Open);
            BinaryReader BReader = new BinaryReader(filestream);
            
            byte[] bfile_len = new byte[4];
            bfile_len[0] = (byte)(file_byte_size >> 24);
            bfile_len[1] = (byte)(file_byte_size >> 16);
            bfile_len[2] = (byte)(file_byte_size >> 8);
            bfile_len[3] = (byte)file_byte_size;
                                   

            System.Threading.Thread.Sleep(500);
            if (!PublicFunction.StarTrans(bfile_len))
            {
                this.BeginInvoke(addShow, new object[] { "发送升级文件长度失败" });
                return;
            }
            this.BeginInvoke(addShow, new object[] { "发送升级文件长度成功" });
            System.Threading.Thread.Sleep(10);

            //发送完整数据包
            for (i = 0; i < package_num; i++)
            {
                BReader.Read(package, 0, 1024);     
                if (!PublicFunction.TranPackage((byte)i, 0x00, 1024, package))
                {
                    BReader.Close();
                    filestream.Close();
                    this.BeginInvoke(addShow, new object[] { "发送升级包 " + i + " 失败" });
                    return;
                }
                this.BeginInvoke(updateBarByStep, new object[] { 1024 });
                this.BeginInvoke(addShow, new object[] { "发送升级包 " + i + " 成功" });
                //System.Threading.Thread.Sleep(1000);
            }

            //发送剩余数据包
            BReader.Read(package, 0, lastpack_len);
            if (!PublicFunction.TranPackage((byte)i, 0x01, (int)(lastpack_len), package))
            {
                BReader.Close();
                filestream.Close();
                this.BeginInvoke(addShow, new object[] { "发送升级包 " + i + " 失败" });
                return;
            }
            this.BeginInvoke(updateBarByStep, new object[] { lastpack_len });
            this.BeginInvoke(addShow, new object[] { "发送升级包 " + i + " 成功" });
            BReader.Close();
            filestream.Close();

            //System.Threading.Thread.Sleep(800);
            if (!PublicFunction.EndUpdate())
            {
                this.BeginInvoke(addShow, new object[] { "升级失败"});
                return;
            }
            this.BeginInvoke(addShow, new object[] { "升级成功" });
        }

        private void UpdateProgressBarByStep(int step)
        {
            PBUpdate.Value += step;
            PBUpdate.Refresh();
        }

        public void AddTxt(string txt)
        {
            TbShow.AppendText(txt + "\r\n");
        }
        
    }
}
